/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Entwicklung
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49 (0)7225-64-0
 * Fax: +49 (0)7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse implementiert ein Objekt vom Typ 'typ.ereignis'.<p>
 * 
 * ber die Online-Attributgruppe 'atg.ereignis' wird der aktuelle Zustand
 * des Ereignisses festgestellt und zur Verfgung gestellt.
 * 
 * ber Getter kann auf die spezifischen Daten zugegriffen werden.
 *
 * @version $Revision: 1.2 $ / $Date: 2012/07/23 06:02:36 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class Ereignis implements ClientReceiverInterface
{
	/**
   * Stellt Methoden fr die Debugausgabe zur Vefgung.
   */
  private static Debug m_debug = Debug.getLogger();
  /**
   * Schnittstellen, die die logische Verbindung zum Datenverteiler reprsentiert. 
   */
	private ClientDavInterface m_connection;
	/**
	 * Schnittstelle zum Zugriff auf Objekte und Eigenschaften eines Datenmodells.
	 */
	private DataModel m_dataModel;
	/**
	 * Systemobjekt Ereignis.
	 */
	private SystemObject m_objekt;	
	/**
	 * Attributgruppe fr den Datenempfang.
	 */
	private AttributeGroup m_atgEreignis;
	/**
	 * Aspekt fr den Datenempfang.
	 */
	private Aspect m_aspZustand;	
	/**
	 * Kennung, ob das Ereignis zeitlich gltig ist.
	 */
	private boolean m_zeitlichGueltig = false;	
	/**
	 * Ereignistyp des Ereignisses.
	 */
	private SystemObject m_ereignisTyp;
	
	/**
	 * Default-Konstruktor ohne weitere Funktion.
	 */
	public Ereignis()
	{		
	}
	
	/**
	 * Konstruktor mit bernahme der Parameter.
	 * 
	 * @param connection Aktuelle Verbindung zum Datenverteiler.
	 * @param objekt Objekt Ereignis.
	 */
	public Ereignis(ClientDavInterface connection, SystemObject objekt)
	{
		m_connection = connection;
		m_dataModel = m_connection.getDataModel();
		m_objekt = objekt;
		
		m_atgEreignis = m_dataModel.getAttributeGroup("atg.ereignis");
		m_aspZustand = m_dataModel.getAspect("asp.zustand");

		m_ereignisTyp = bestimmeEreignisTyp();
		
		// Daten zum Empfang anmelden		
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgEreignis, m_aspZustand);

	}
	
	/**
	 * Bestimmt den Ereignistyp des Ereignisses.
	 * 
	 * @return Objekt Ereignistyp.
	 */
	private SystemObject bestimmeEreignisTyp()
	{
		if (m_objekt != null)
		{
			Data data = m_objekt.getConfigurationData(m_dataModel.getAttributeGroup("atg.ereignisEigenschaften"));
			if (data != null)
			{
				return data.getReferenceValue("EreignisTypReferenz").getSystemObject();
			}
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
	 */
	@Override
	public void update(ResultData results[])
	{		
		for (int i = 0; i < results.length; i++)
		{
			ResultData resultData = results[i];
			
//			System.out.println("--------------> Status Ereignis empfangen");
//			System.out.println("--------------> " + resultData);
			
      if (resultData.getDataDescription().getAttributeGroup().equals(m_atgEreignis))
      {
      	if (resultData.hasData())
      	{
      		Data data = resultData.getData();
      		
      		m_zeitlichGueltig = data.getScaledValue("zeitlichGltig").getValueText().equals("Ja");
      	}
      	else
      	{
      		m_zeitlichGueltig = false;
      	}
      }
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() 
	{
		StringBuffer string = new StringBuffer();
		
		string.append(" " + m_objekt + " eTyp " + m_ereignisTyp + " zeitlichGltig = " + m_zeitlichGueltig); 					
		return string.toString();
  }
	
	/**
   * Fr ein Objekt Attributgruppe/Aspekt Kombination am Datenverteiler anmelden.
   * 
   * @param instanz Instanz.
   * @param objekt Objekt.
   * @param atg Attributgruppe.
   * @param asp Aspekt.
   * @return true = Anmeldung erfolgreich, false = nicht erfolgreich.
   */
  public boolean empfaengerAnmeldenObjekt(ClientReceiverInterface instanz, SystemObject objekt, AttributeGroup atg, Aspect asp)
  {
    DataDescription dd;
    boolean ok = false;

    m_debug.fine(("DatenAnmeldenObjekt " + atg.getPid() + " " + asp.getPid() + " fr " + objekt));
    
    try
    {  
    	dd = new DataDescription(atg, asp);

    	m_connection.subscribeReceiver(instanz, objekt, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
        
    	ok = true;
    } 
    catch (DataNotSubscribedException e)
    {
      m_debug.error(("Daten nicht zum Empfangen angemeldet " + e.getMessage()));
    } 
    
    return (ok);
  }

	/**
	 * Prft, ob das Ereignis zeitlich gltig ist.
	 * 
	 * @return true wenn Ereignis zeitlich gltig ist, ansonsten false.
	 */
	public boolean isZeitlichGueltig()
	{
		return m_zeitlichGueltig;
	}

	/**
	 * Gibt den Ereignistyp des Objekts zurck.
	 * 
	 * @return Ereignisyyp des Objekts.
	 */
	public SystemObject getEreignisTyp()
	{
		return m_ereignisTyp;
	}

}
