/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Entwicklung
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49 (0)7225-64-0
 * Fax: +49 (0)7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.stau;

import java.util.List;

/**
 * Klasse stellt Methoden zur Klassifizierung und Zusammenfassung von Staukriterien bereit. 
 *
 * @version $Revision: 1.2 $ / $Date: 2012/07/23 06:02:35 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class StaukriteriumKlassifizierung
{

	/**
	 * Ermittelt die Zugegrigkeit des Einschaltkriteriums.
	 * 
	 * @param einkrit Einschaltkriterium.
	 * @param auskrit Ausschaltkriterium.
	 * @return Zugehrigkeit des Einschaltkriteriums.
	 */
	public static float zugehoerigkeitStau(boolean einkrit, boolean auskrit, boolean zustandVerkehrlichAlt)
	{		
		if (einkrit)
		{
			return 1.0F;
		}
		else if ((!einkrit) && (!auskrit))
		{
			if (zustandVerkehrlichAlt)
			{
			  return 0.5F;
			}
		}
				
		return 0.0F;		
	}

	/**
	 * Ermittelt die Zugegrigkeit des Ausschaltkriteriums.
	 * 
	 * @param einkrit Einschaltkriterium.
	 * @param auskrit Ausschaltkriterium.
	 * @return Zugehrigkeit des Ausschaltkriteriums.
	 */
	public static float zugehoerigkeitNichtStau(boolean einkrit, boolean auskrit, boolean zustandVerkehrlichAlt)
	{		
		if (auskrit)
		{
			return 1.0F;
		}
		else if ((!einkrit) && (!auskrit))
		{
			if (!zustandVerkehrlichAlt)
			{
				return 0.5F;
			}
		}
		
		return 0.0F;		
	}

	/**
	 * Zusammenfassung der Staukriterien.
	 * 
	 * @param staukriterium1 Daten Staukriterium 1.
	 * @param staukriterium2 Daten Staukriterium 2.
	 * @param staukriterium3 Daten Staukriterium 3.
	 * @param staukriterium4 Daten Staukriterium 4.
	 * @return Ergebnis der Zusammenfassung.
	 */
	public static StaukriteriumZusammenfassung staukriteriumZusammenfassung(Staukriterium staukriterium1, Staukriterium staukriterium2, Staukriterium staukriterium3, Staukriterium staukriterium4 )
	{
		float psn = 0.0F;
		float pnsn = 0.0F; 
		float gs =  0.0F;
		float gns =  0.0F;
		
		float pns = 0.0F;
		float ps = 0.0F;
		float _gs = 0.0F;
		float _gns = 0.0F;
		float _vs = 0.0F;
		float _vns = 0.0F;
		
		pns += (staukriterium1.getVertrauenswrdigkeitNichtStau() * staukriterium1.getZugehrigkeitNichtStau() * staukriterium1.getGteNichtStau());
		pns += (staukriterium2.getVertrauenswrdigkeitNichtStau() * staukriterium2.getZugehrigkeitNichtStau() * staukriterium2.getGteNichtStau());
		pns += (staukriterium3.getVertrauenswrdigkeitNichtStau() * staukriterium3.getZugehrigkeitNichtStau() * staukriterium3.getGteNichtStau());
		pns += (staukriterium4.getVertrauenswrdigkeitNichtStau() * staukriterium4.getZugehrigkeitNichtStau() * staukriterium4.getGteNichtStau());

		_gs += (staukriterium1.getVertrauenswrdigkeitNichtStau() * staukriterium1.getGteNichtStau());
		_gs += (staukriterium2.getVertrauenswrdigkeitNichtStau() * staukriterium2.getGteNichtStau());
		_gs += (staukriterium3.getVertrauenswrdigkeitNichtStau() * staukriterium3.getGteNichtStau());
		_gs += (staukriterium4.getVertrauenswrdigkeitNichtStau() * staukriterium4.getGteNichtStau());

		_vs += staukriterium1.getVertrauenswrdigkeitNichtStau();
		_vs += staukriterium2.getVertrauenswrdigkeitNichtStau();
		_vs += staukriterium3.getVertrauenswrdigkeitNichtStau();
		_vs += staukriterium4.getVertrauenswrdigkeitNichtStau();
		
		ps += (staukriterium1.getVertrauenswrdigkeitStau() * staukriterium1.getZugehrigkeitStau() * staukriterium1.getGteStau());
		ps += (staukriterium2.getVertrauenswrdigkeitStau() * staukriterium2.getZugehrigkeitStau() * staukriterium2.getGteStau());
		ps += (staukriterium3.getVertrauenswrdigkeitStau() * staukriterium3.getZugehrigkeitStau() * staukriterium3.getGteStau());
		ps += (staukriterium4.getVertrauenswrdigkeitStau() * staukriterium4.getZugehrigkeitStau() * staukriterium4.getGteStau());

		_gns += (staukriterium1.getVertrauenswrdigkeitStau() * staukriterium1.getGteStau());
		_gns += (staukriterium2.getVertrauenswrdigkeitStau() * staukriterium2.getGteStau());
		_gns += (staukriterium3.getVertrauenswrdigkeitStau() * staukriterium3.getGteStau());
		_gns += (staukriterium4.getVertrauenswrdigkeitStau() * staukriterium4.getGteStau());

		_vns += staukriterium1.getVertrauenswrdigkeitStau();
		_vns += staukriterium2.getVertrauenswrdigkeitStau();
		_vns += staukriterium3.getVertrauenswrdigkeitStau();
		_vns += staukriterium1.getVertrauenswrdigkeitStau();
		
		
		// Normierung
		if ((ps + pns) > 0)
		{
			psn = ps / (ps + pns); 
			pnsn = pns / (ps + pns); 
		}		

		// Gte
		if (_vs > 0)
			gs = _gs / _vs;

		if (_vns > 0)
			gns = _gns / _vns;
		
		// Daten in Ergebnisklasse speichern
		
		StaukriteriumZusammenfassung zusammenfassung = new StaukriteriumZusammenfassung();
		
		zusammenfassung.getNichtStau().setPn(pnsn);
		zusammenfassung.getNichtStau().setP(pns);
		zusammenfassung.getNichtStau().setG(gns);

		zusammenfassung.getStau().setPn(psn);
		zusammenfassung.getStau().setP(ps);
		zusammenfassung.getStau().setG(gs);

		if (psn > pnsn)
			zusammenfassung.setStaukriterium("Stau");
		else
			zusammenfassung.setStaukriterium("Nicht-Stau");
			
		System.out.println("Zusammenfassung Staukriterien " + zusammenfassung);
		
		return zusammenfassung;
	}

	/**
	 * Zusammenfassung der Staukriterien.
	 * 
	 * @param staukriterien Daten der Staukriterien.
	 * @return Ergebnis der Zusammenfassung.
	 */
	public static StaukriteriumZusammenfassung staukriteriumZusammenfassung(List<Staukriterium> staukriterien)
	{
		float psn = 0.0F;
		float pnsn = 0.0F; 
		float gs =  0.0F;
		float gns =  0.0F;
		
		float pns = 0.0F;
		float ps = 0.0F;
		float _gs = 0.0F;
		float _gns = 0.0F;
		float _vs = 0.0F;
		float _vns = 0.0F;
		
		String _staukriterium = "Nicht-Stau";
		
		
		for (int i = 0; i < staukriterien.size(); i++)
		{
			Staukriterium staukriterium = staukriterien.get(i);
			
			pns += (staukriterium.getVertrauenswrdigkeitNichtStau() * staukriterium.getZugehrigkeitNichtStau() * staukriterium.getGteNichtStau());
			_gns += (staukriterium.getVertrauenswrdigkeitNichtStau() * staukriterium.getGteNichtStau());
			_vns += staukriterium.getVertrauenswrdigkeitNichtStau();
			
			ps += (staukriterium.getVertrauenswrdigkeitStau() * staukriterium.getZugehrigkeitStau() * staukriterium.getGteStau());
			_gs += (staukriterium.getVertrauenswrdigkeitStau() * staukriterium.getGteStau());
			_vs += staukriterium.getVertrauenswrdigkeitStau();
		}
				
		// Normierung
		if ((ps + pns) > 0)
		{
			psn = ps / (ps + pns); 
			pnsn = pns / (ps + pns); 
		}		

		// Gte
		if (_vs > 0)
			gs = _gs / _vs;

		if (_vns > 0)
			gns = _gns / _vns;

		// Staukriterium
		if (psn > pnsn)
			_staukriterium = "Stau";

		// Daten in Ergebnisklasse speichern
		
		StaukriteriumZusammenfassung zusammenfassung = new StaukriteriumZusammenfassung();
		
		zusammenfassung.setStaukriterium(_staukriterium);
		
		zusammenfassung.getNichtStau().setPn(pnsn);		
		zusammenfassung.getNichtStau().setP(pns);
		zusammenfassung.getNichtStau().setG(gns);

		zusammenfassung.getStau().setPn(psn);
		zusammenfassung.getStau().setP(ps);
		zusammenfassung.getStau().setG(gs);
					
		return zusammenfassung;
	}

}
